{{
Purpose of PASMstore is to have an environment in which PASM code can be created,
tested and then stored on an SD card. Of course it can be adapted to provide this
functionality for any kind of mass storage.
These PASM code snippets can then be loaded by other programs. This is good for
dynamic program reload (e.g. drivers) with a small memory footprint. In this way the whole 32k of
HUB RAM can be used for other application code or data.

This program uses the SD card driver available at the ObjectExchange.

Please make sure that you modify the SDCardPinBase to match your SD card connection.
Please note the comments in SDSPIQASM on how to connect the SD card if you don't
have a board including a SD card slot.

I like to see what a program is doing. That's why I used the serial driver to
show some short messages. For bigger libraries of PASM it's useful for debugging
of course.

In a later version all the FullDuplexSerial code and all the SD card code should
be included directly in this file. This way the PASMstore can be run with F10 whereas
the BigProgram can be run with Ctrl + F10.

The content of a *.cog file is simply the length of the PASM code, which is stored in
the first 4 bytes. (Yes, I know that this is by far to long for the propeller, but
the propeller is a 32 bit controller ;o)
Following bytes contain the machine code.
}}
CON
        _clkmode        = xtal1 + pll8x
        _xinfreq        = 10_000_000

        SDCardPinBase   = 19
        
obj
   term: "FullDuplexSerial"
   sdfat: "fsrw"
   
var
        long          led_pin, dt, length
        
pub PASMstore
  ' initialize the terminal connection and wait until terminal gets switched on
  term.start( 31, 30, 0, 115200 )
  ' init the SD card driver
  sdfat.mount( SDCardPinBase )
  ' wait a while, so the terminal can be re-connected
  waitcnt( clkfreq*10+cnt )

  ' show which program is running
  term.str( string("PASMstore",13) )

  ' this one starts the movable to test it
  term.str( string("Running 'LED toggler'",13) )
  led_pin:=1
  dt:=80_000_000
  cognew(@movable, @led_pin)
  repeat until led_pin==0

  ' here we save the movable to the SD-card
  ' this code is only switched on, if the test was successfull and the code can be saved
  term.str( string("Storing 'LED toggler'",13) )
  sdfat.popen(string("ledtoggl.cog"), "w")
  ' calculate the length of the PASM-code
  length:=@movable_end-@movable
  ' first write the lenght and then the code
  sdfat.pwrite(@length, 4)
  sdfat.pwrite(@movable, length)
  sdfat.pclose
  
          
dat
              org 0
movable                                         ' this points to the start of PASM-code
              mov       hub_pin, par            ' get adresses of the HUB parameter buffer
              add       hub_wait, par
pin           rdlong    pin, hub_pin            ' read the pin reusing this RAM
waittime      rdlong    waittime, hub_wait      ' read the wait time reusing this RAM            
              wrlong    inb, hub_pin
                            
time          mov       dira, pin               ' set the given pin to output
              mov       time, cnt               ' calculate the first waitcnt-value
              add       time, waittime
              
toggle        xor       outa, pin               ' toggle the pin
              waitcnt   time, waittime          ' wait for the given time
              jmp       #toggle                 ' and repeat endless
               

hub_pin       long      0
hub_wait      long      4
movable_end   long      1                       ' this will point to the end of the PASM-code

                               